defmodule Chronoscope.NTS.KeyEstablishmentClientTest do
  use Chronoscope.Case

  alias Chronoscope.NTS.SSLMock
  alias Chronoscope.NTS.KeyEstablishmentRequest

  import Chronoscope.NTS.KeyEstablishmentClient
  import Mox

  setup :verify_on_exit!

  @timeout 3000

  defp peercert() do
    :secp256r1
    |> X509.PrivateKey.new_ec()
    |> X509.Certificate.self_signed("/C=US/ST=CA/L=San Francisco/O=Acme/CN=Test")
    |> X509.Certificate.to_der()
  end

  defp send_ssl_response(response) do
    send(self(), {:ssl, nil, response})
    :ok
  end

  describe "Chronoscope.NTS.KeyEstablishmentClient.key_establishment()" do
    test "handles an empty response" do
      request = KeyEstablishmentRequest.create()

      SSLMock
      |> expect(:connect, fn ~c"localhost", 2222, _tls_options, @timeout -> {:ok, :socket} end)
      |> expect(:send, fn :socket, ^request -> send_ssl_response([]) end)
      |> expect(:peercert, fn :socket -> {:ok, peercert()} end)
      |> expect(:close, fn :socket -> :ok end)

      assert {:ok, %{cert_expiration: _expiration}} = key_establishment(%{host: "localhost", port: 2222})
    end

    test "sends the correct TLS options" do
      SSLMock
      |> expect(:connect, fn ~c"localhost", 2222, tls_options, @timeout ->
        assert tls_options[:alpn_advertised_protocols] == ["ntske/1"]
        {:ok, :socket}
      end)
      |> expect(:send, fn :socket, _request -> send_ssl_response([]) end)
      |> expect(:peercert, fn :socket -> {:ok, peercert()} end)
      |> expect(:close, fn :socket -> :ok end)

      key_establishment(%{host: "localhost", port: 2222})
    end
  end
end
